@@ -4,8 +4,8 @@ use syn::parse::Error;
 
 use crate::field_info::{FieldBuilderAttr, FieldInfo};
 use crate::util::{
-    empty_type, empty_type_tuple, expr_to_single_string, make_punctuated_single, modify_types_generics_hack,
-    path_to_single_string, strip_raw_ident_prefix, type_tuple,
+    empty_type, empty_type_tuple, expr_to_single_string, first_visibility, make_punctuated_single, modify_types_generics_hack,
+    path_to_single_string, public_visibility, strip_raw_ident_prefix, type_tuple,
 };
 
 #[derive(Debug)]
@@ -28,7 +28,11 @@ impl<'a> StructInfo<'a> {
 
     pub fn new(ast: &'a syn::DeriveInput, fields: impl Iterator<Item = &'a syn::Field>) -> Result<StructInfo<'a>, Error> {
         let builder_attr = TypeBuilderAttr::new(&ast.attrs)?;
-        let builder_name = strip_raw_ident_prefix(format!("{}Builder", ast.ident));
+        let builder_name = builder_attr
+            .builder_type
+            .get_name()
+            .map(|name| strip_raw_ident_prefix(name.to_string()))
+            .unwrap_or_else(|| strip_raw_ident_prefix(format!("{}Builder", ast.ident)));
         Ok(StructInfo {
             vis: &ast.vis,
             name: &ast.ident,
@@ -44,28 +48,26 @@ impl<'a> StructInfo<'a> {
         })
     }
 
-    fn modify_generics<F: FnMut(&mut syn::Generics)>(&self, mut mutator: F) -> syn::Generics {
-        let mut generics = self.generics.clone();
-        mutator(&mut generics);
-        generics
-    }
-
     pub fn builder_creation_impl(&self) -> Result<TokenStream, Error> {
         let StructInfo {
-            ref vis,
+            vis,
             ref name,
             ref builder_name,
             ..
         } = *self;
         let (impl_generics, ty_generics, where_clause) = self.generics.split_for_impl();
-        let all_fields_param =
-            syn::GenericParam::Type(syn::Ident::new("TypedBuilderFields", proc_macro2::Span::call_site()).into());
-        let b_generics = self.modify_generics(|g| {
-            g.params.insert(0, all_fields_param.clone());
-        });
         let empties_tuple = type_tuple(self.included_fields().map(|_| empty_type()));
+        let mut all_fields_param_type: syn::TypeParam =
+            syn::Ident::new("TypedBuilderFields", proc_macro2::Span::call_site()).into();
+        let all_fields_param = syn::GenericParam::Type(all_fields_param_type.clone());
+        all_fields_param_type.default = Some(syn::Type::Tuple(empties_tuple.clone()));
+        let b_generics = {
+            let mut generics = self.generics.clone();
+            generics.params.push(syn::GenericParam::Type(all_fields_param_type));
+            generics
+        };
         let generics_with_empty = modify_types_generics_hack(&ty_generics, |args| {
-            args.insert(0, syn::GenericArgument::Type(empties_tuple.clone().into()));
+            args.push(syn::GenericArgument::Type(empties_tuple.clone().into()));
         });
         let phantom_generics = self.generics.params.iter().map(|param| match param {
             syn::GenericParam::Lifetime(lifetime) => {
@@ -80,15 +82,20 @@ impl<'a> StructInfo<'a> {
                 quote!()
             }
         });
-        let builder_method_doc = if let Some(ref doc) = self.builder_attr.builder_method_doc {
-            quote!(#doc)
-        } else {
-            let doc = format!(
+
+        let builder_method_name = self.builder_attr.builder_method.get_name().unwrap_or_else(|| quote!(builder));
+        let builder_method_visibility = first_visibility(&[
+            self.builder_attr.builder_method.vis.as_ref(),
+            self.builder_attr.builder_type.vis.as_ref(),
+            Some(vis),
+        ]);
+        let builder_method_doc = self.builder_attr.builder_method.get_doc_or(|| {
+            format!(
                 "
-                    Create a builder for building `{name}`.
-                    On the builder, call {setters} to set the values of the fields.
-                    Finally, call `.build()` to create the instance of `{name}`.
-                    ",
+                Create a builder for building `{name}`.
+                On the builder, call {setters} to set the values of the fields.
+                Finally, call `.build()` to create the instance of `{name}`.
+                ",
                 name = self.name,
                 setters = {
                     let mut result = String::new();
@@ -107,19 +114,17 @@ impl<'a> StructInfo<'a> {
                     }
                     result
                 }
-            );
-            quote!(#doc)
-        };
+            )
+        });
+
+        let builder_type_visibility = first_visibility(&[self.builder_attr.builder_type.vis.as_ref(), Some(vis)]);
         let builder_type_doc = if self.builder_attr.doc {
-            if let Some(ref doc) = self.builder_attr.builder_type_doc {
-                quote!(#[doc = #doc])
-            } else {
-                let doc = format!(
+            self.builder_attr.builder_type.get_doc_or(|| {
+                format!(
                     "Builder for [`{name}`] instances.\n\nSee [`{name}::builder()`] for more info.",
                     name = name
-                );
-                quote!(#[doc = #doc])
-            }
+                )
+            })
         } else {
             quote!(#[doc(hidden)])
         };
@@ -134,9 +139,9 @@ impl<'a> StructInfo<'a> {
 
         Ok(quote! {
             impl #impl_generics #name #ty_generics #where_clause {
-                #[doc = #builder_method_doc]
+                #builder_method_doc
                 #[allow(dead_code, clippy::default_trait_access)]
-                #vis fn builder() -> #builder_name #generics_with_empty {
+                #builder_method_visibility fn #builder_method_name() -> #builder_name #generics_with_empty {
                     #builder_name {
                         fields: #empties_tuple,
                         phantom: ::core::default::Default::default(),
@@ -147,7 +152,7 @@ impl<'a> StructInfo<'a> {
             #[must_use]
             #builder_type_doc
             #[allow(dead_code, non_camel_case_types, non_snake_case)]
-            #vis struct #builder_name #b_generics {
+            #builder_type_visibility struct #builder_name #b_generics {
                 fields: #all_fields_param,
                 phantom: (#( #phantom_generics ),*),
             }
@@ -225,18 +230,14 @@ impl<'a> StructInfo<'a> {
             .collect();
         let mut target_generics_tuple = empty_type_tuple();
         let mut ty_generics_tuple = empty_type_tuple();
-        let generics = self.modify_generics(|g| {
-            let index_after_lifetime_in_generics = g
-                .params
-                .iter()
-                .filter(|arg| matches!(arg, syn::GenericParam::Lifetime(_)))
-                .count();
+        let generics = {
+            let mut generics = self.generics.clone();
             for f in self.included_fields() {
                 if f.ordinal == field.ordinal {
                     ty_generics_tuple.elems.push_value(empty_type());
                     target_generics_tuple.elems.push_value(f.tuplized_type_ty_param());
                 } else {
-                    g.params.insert(index_after_lifetime_in_generics, f.generic_ty_param());
+                    generics.params.push(f.generic_ty_param());
                     let generic_argument: syn::Type = f.type_ident();
                     ty_generics_tuple.elems.push_value(generic_argument.clone());
                     target_generics_tuple.elems.push_value(generic_argument);
@@ -244,20 +245,11 @@ impl<'a> StructInfo<'a> {
                 ty_generics_tuple.elems.push_punct(Default::default());
                 target_generics_tuple.elems.push_punct(Default::default());
             }
-        });
+            generics
+        };
         let mut target_generics = ty_generics.clone();
-        let index_after_lifetime_in_generics = target_generics
-            .iter()
-            .filter(|arg| matches!(arg, syn::GenericArgument::Lifetime(_)))
-            .count();
-        target_generics.insert(
-            index_after_lifetime_in_generics,
-            syn::GenericArgument::Type(target_generics_tuple.into()),
-        );
-        ty_generics.insert(
-            index_after_lifetime_in_generics,
-            syn::GenericArgument::Type(ty_generics_tuple.into()),
-        );
+        target_generics.push(syn::GenericArgument::Type(target_generics_tuple.into()));
+        ty_generics.push(syn::GenericArgument::Type(ty_generics_tuple.into()));
         let (impl_generics, _, where_clause) = generics.split_for_impl();
         let doc = match field.builder_attr.setter.doc {
             Some(ref doc) => quote!(#[doc = #doc]),
@@ -267,10 +259,9 @@ impl<'a> StructInfo<'a> {
         // NOTE: both auto_into and strip_option affect `arg_type` and `arg_expr`, but the order of
         // nesting is different so we have to do this little dance.
         let arg_type = if field.builder_attr.setter.strip_option.is_some() && field.builder_attr.setter.transform.is_none() {
-            let internal_type = field
+            field
                 .type_from_inside_option()
-                .ok_or_else(|| Error::new_spanned(&field_type, "can't `strip_option` - field is not `Option<...>`"))?;
-            internal_type
+                .ok_or_else(|| Error::new_spanned(field_type, "can't `strip_option` - field is not `Option<...>`"))?
         } else {
             field_type
         };
@@ -358,12 +349,8 @@ impl<'a> StructInfo<'a> {
             })
             .collect();
         let mut builder_generics_tuple = empty_type_tuple();
-        let generics = self.modify_generics(|g| {
-            let index_after_lifetime_in_generics = g
-                .params
-                .iter()
-                .filter(|arg| matches!(arg, syn::GenericParam::Lifetime(_)))
-                .count();
+        let generics = {
+            let mut generics = self.generics.clone();
             for f in self.included_fields() {
                 if f.builder_attr.default.is_some() {
                     // `f` is not mandatory - it does not have it's own fake `build` method, so `field` will need
@@ -373,7 +360,7 @@ impl<'a> StructInfo<'a> {
                         "`required_field_impl` called for optional field {}",
                         field.name
                     );
-                    g.params.insert(index_after_lifetime_in_generics, f.generic_ty_param());
+                    generics.params.push(f.generic_ty_param());
                     builder_generics_tuple.elems.push_value(f.type_ident());
                 } else if f.ordinal < field.ordinal {
                     // Only add a `build` method that warns about missing `field` if `f` is set. If `f` is not set,
@@ -385,22 +372,16 @@ impl<'a> StructInfo<'a> {
                     // `f` appears later in the argument list after `field`, so if they are both missing we will
                     // show a warning for `field` and not for `f` - which means this warning should appear whether
                     // or not `f` is set.
-                    g.params.insert(index_after_lifetime_in_generics, f.generic_ty_param());
+                    generics.params.push(f.generic_ty_param());
                     builder_generics_tuple.elems.push_value(f.type_ident());
                 }
 
                 builder_generics_tuple.elems.push_punct(Default::default());
             }
-        });
+            generics
+        };
 
-        let index_after_lifetime_in_generics = builder_generics
-            .iter()
-            .filter(|arg| matches!(arg, syn::GenericArgument::Lifetime(_)))
-            .count();
-        builder_generics.insert(
-            index_after_lifetime_in_generics,
-            syn::GenericArgument::Type(builder_generics_tuple.into()),
-        );
+        builder_generics.push(syn::GenericArgument::Type(builder_generics_tuple.into()));
         let (impl_generics, _, where_clause) = generics.split_for_impl();
         let (_, ty_generics, _) = self.generics.split_for_impl();
 
@@ -414,6 +395,9 @@ impl<'a> StructInfo<'a> {
         );
         let early_build_error_message = format!("Missing required field {}", field_name);
 
+        let build_method_name = self.build_method_name();
+        let build_method_visibility = self.build_method_visibility();
+
         quote! {
             #[doc(hidden)]
             #[allow(dead_code, non_camel_case_types, non_snake_case)]
@@ -424,13 +408,21 @@ impl<'a> StructInfo<'a> {
                 #[deprecated(
                     note = #early_build_error_message
                 )]
-                pub fn build(self, _: #early_build_error_type_name) -> #name #ty_generics {
+                #build_method_visibility fn #build_method_name(self, _: #early_build_error_type_name) -> #name #ty_generics {
                     panic!();
                 }
             }
         }
     }
 
+    fn build_method_name(&self) -> TokenStream {
+        self.builder_attr.build_method.common.get_name().unwrap_or(quote!(build))
+    }
+
+    fn build_method_visibility(&self) -> TokenStream {
+        first_visibility(&[self.builder_attr.build_method.common.vis.as_ref(), Some(&public_visibility())])
+    }
+
     pub fn build_method_impl(&self) -> TokenStream {
         let StructInfo {
             ref name,
@@ -438,12 +430,8 @@ impl<'a> StructInfo<'a> {
             ..
         } = *self;
 
-        let generics = self.modify_generics(|g| {
-            let index_after_lifetime_in_generics = g
-                .params
-                .iter()
-                .filter(|arg| matches!(arg, syn::GenericParam::Lifetime(_)))
-                .count();
+        let generics = {
+            let mut generics = self.generics.clone();
             for field in self.included_fields() {
                 if field.builder_attr.default.is_some() {
                     let trait_ref = syn::TraitBound {
@@ -463,28 +451,26 @@ impl<'a> StructInfo<'a> {
                     };
                     let mut generic_param: syn::TypeParam = field.generic_ident.clone().into();
                     generic_param.bounds.push(trait_ref.into());
-                    g.params.insert(index_after_lifetime_in_generics, generic_param.into());
+                    generics.params.push(generic_param.into());
                 }
             }
-        });
+            generics
+        };
         let (impl_generics, _, _) = generics.split_for_impl();
 
         let (_, ty_generics, where_clause) = self.generics.split_for_impl();
 
         let modified_ty_generics = modify_types_generics_hack(&ty_generics, |args| {
-            args.insert(
-                0,
-                syn::GenericArgument::Type(
-                    type_tuple(self.included_fields().map(|field| {
-                        if field.builder_attr.default.is_some() {
-                            field.type_ident()
-                        } else {
-                            field.tuplized_type_ty_param()
-                        }
-                    }))
-                    .into(),
-                ),
-            );
+            args.push(syn::GenericArgument::Type(
+                type_tuple(self.included_fields().map(|field| {
+                    if field.builder_attr.default.is_some() {
+                        field.type_ident()
+                    } else {
+                        field.tuplized_type_ty_param()
+                    }
+                }))
+                .into(),
+            ));
         });
 
         let descructuring = self.included_fields().map(|f| f.name);
@@ -508,44 +494,38 @@ impl<'a> StructInfo<'a> {
             }
         });
         let field_names = self.fields.iter().map(|field| field.name);
-        let doc = if self.builder_attr.doc {
-            if let Some(ref doc) = self.builder_attr.build_method_doc {
-                quote!(#[doc = #doc])
-            } else {
-                // I'd prefer “a” or “an” to “its”, but determining which is grammatically
-                // correct is roughly impossible.
-                let doc = format!("Finalise the builder and create its [`{}`] instance", name);
-                quote!(#[doc = #doc])
-            }
+
+        let build_method_name = self.build_method_name();
+        let build_method_visibility = self.build_method_visibility();
+        let build_method_doc = if self.builder_attr.doc {
+            self.builder_attr
+                .build_method
+                .common
+                .get_doc_or(|| format!("Finalise the builder and create its [`{}`] instance", name))
         } else {
             quote!()
         };
+        let (build_method_generic, output_type, build_method_where_clause) = match &self.builder_attr.build_method.into {
+            IntoSetting::NoConversion => (None, quote!(#name #ty_generics), None),
+            IntoSetting::GenericConversion => (
+                Some(quote!(<__R>)),
+                quote!(__R),
+                Some(quote!(where #name #ty_generics: Into<__R>)),
+            ),
+            IntoSetting::TypeConversionToSpecificType(into) => (None, quote!(#into), None),
+        };
 
-        let build_method_name = self
-            .builder_attr
-            .build_method
-            .name
-            .as_ref()
-            .map(|name| quote!(#name))
-            .unwrap_or(quote!(build));
-        let visibility = self
-            .builder_attr
-            .build_method
-            .vis
-            .as_ref()
-            .map(|v| quote!(#v))
-            .unwrap_or(quote!(pub));
         quote!(
             #[allow(dead_code, non_camel_case_types, missing_docs)]
             impl #impl_generics #builder_name #modified_ty_generics #where_clause {
-                #doc
+                #build_method_doc
                 #[allow(clippy::default_trait_access)]
-                #visibility fn #build_method_name(self) -> #name #ty_generics {
+                #build_method_visibility fn #build_method_name #build_method_generic (self) -> #output_type #build_method_where_clause {
                     let ( #(#descructuring,)* ) = self.fields;
                     #( #assignments )*
                     #name {
                         #( #field_names ),*
-                    }
+                    }.into()
                 }
             }
         )
@@ -553,11 +533,12 @@ impl<'a> StructInfo<'a> {
 }
 
 #[derive(Debug, Default, Clone)]
-pub struct BuildMethodSettings {
+pub struct CommonDeclarationSettings {
     pub vis: Option<syn::Visibility>,
     pub name: Option<syn::Expr>,
+    pub doc: Option<syn::Expr>,
 }
-impl BuildMethodSettings {
+impl CommonDeclarationSettings {
     fn apply_meta(&mut self, expr: syn::Expr) -> Result<(), Error> {
         match expr {
             syn::Expr::Assign(assign) => {
@@ -579,12 +560,85 @@ impl BuildMethodSettings {
                         self.name = Some(*assign.right);
                         Ok(())
                     }
+                    "doc" => {
+                        self.doc = Some(*assign.right);
+                        Ok(())
+                    }
                     _ => Err(Error::new_spanned(&assign, format!("Unknown parameter {:?}", name))),
                 }
             }
             _ => Err(Error::new_spanned(expr, "Expected (<...>=<...>)")),
         }
     }
+
+    fn get_name(&self) -> Option<TokenStream> {
+        self.name.as_ref().map(|name| quote!(#name))
+    }
+
+    fn get_doc_or(&self, gen_doc: impl FnOnce() -> String) -> TokenStream {
+        if let Some(ref doc) = self.doc {
+            quote!(#[doc = #doc])
+        } else {
+            let doc = gen_doc();
+            quote!(#[doc = #doc])
+        }
+    }
+}
+
+/// Setting of the `into` argument.
+#[derive(Debug, Clone)]
+pub enum IntoSetting {
+    /// Do not run any conversion on the built value.
+    NoConversion,
+    /// Convert the build value into the generic parameter passed to the `build` method.
+    GenericConversion,
+    /// Convert the build value into a specific type specified in the attribute.
+    TypeConversionToSpecificType(syn::ExprPath),
+}
+
+impl Default for IntoSetting {
+    fn default() -> Self {
+        Self::NoConversion
+    }
+}
+
+#[derive(Debug, Default, Clone)]
+pub struct BuildMethodSettings {
+    pub common: CommonDeclarationSettings,
+
+    /// Whether to convert the builded type into another while finishing the build.
+    pub into: IntoSetting,
+}
+
+impl BuildMethodSettings {
+    fn apply_meta(&mut self, expr: syn::Expr) -> Result<(), Error> {
+        match &expr {
+            syn::Expr::Assign(assign) => {
+                let name =
+                    expr_to_single_string(&assign.left).ok_or_else(|| Error::new_spanned(&assign.left, "Expected identifier"))?;
+                if name.as_str() == "into" {
+                    let expr_path = match assign.right.as_ref() {
+                        syn::Expr::Path(expr_path) => expr_path,
+                        _ => return Err(Error::new_spanned(&assign.right, "Expected path expression type")),
+                    };
+                    self.into = IntoSetting::TypeConversionToSpecificType(expr_path.clone());
+                    Ok(())
+                } else {
+                    self.common.apply_meta(expr)
+                }
+            }
+            syn::Expr::Path(path) => {
+                let name = path_to_single_string(&path.path).ok_or_else(|| Error::new_spanned(path, "Expected identifier"))?;
+                if name.as_str() == "into" {
+                    self.into = IntoSetting::GenericConversion;
+                    Ok(())
+                } else {
+                    self.common.apply_meta(expr)
+                }
+            }
+            _ => self.common.apply_meta(expr),
+        }
+    }
 }
 
 #[derive(Debug, Default)]
@@ -592,20 +646,15 @@ pub struct TypeBuilderAttr {
     /// Whether to show docs for the `TypeBuilder` type (rather than hiding them).
     pub doc: bool,
 
+    /// Customize builder method, ex. visibility, name
+    pub builder_method: CommonDeclarationSettings,
+
+    /// Customize builder type, ex. visibility, name
+    pub builder_type: CommonDeclarationSettings,
+
     /// Customize build method, ex. visibility, name
     pub build_method: BuildMethodSettings,
 
-    /// Docs on the `Type::builder()` method.
-    pub builder_method_doc: Option<syn::Expr>,
-
-    /// Docs on the `TypeBuilder` type. Specifying this implies `doc`, but you can just specify
-    /// `doc` instead and a default value will be filled in here.
-    pub builder_type_doc: Option<syn::Expr>,
-
-    /// Docs on the `TypeBuilder.build()` method. Specifying this implies `doc`, but you can just
-    /// specify `doc` instead and a default value will be filled in here.
-    pub build_method_doc: Option<syn::Expr>,
-
     pub field_defaults: FieldBuilderAttr,
 }
 
@@ -637,6 +686,10 @@ impl TypeBuilderAttr {
             }
         }
 
+        if result.builder_type.doc.is_some() || result.build_method.common.doc.is_some() {
+            result.doc = true;
+        }
+
         Ok(result)
     }
 
@@ -645,21 +698,20 @@ impl TypeBuilderAttr {
             syn::Expr::Assign(assign) => {
                 let name =
                     expr_to_single_string(&assign.left).ok_or_else(|| Error::new_spanned(&assign.left, "Expected identifier"))?;
+
+                let gen_structure_depracation_error = |put_under: &str, new_name: &str| {
+                    Error::new_spanned(
+                        &assign.left,
+                        format!(
+                            "`{} = \"...\"` is deprecated - use `{}({} = \"...\")` instead",
+                            name, put_under, new_name
+                        ),
+                    )
+                };
                 match name.as_str() {
-                    "builder_method_doc" => {
-                        self.builder_method_doc = Some(*assign.right);
-                        Ok(())
-                    }
-                    "builder_type_doc" => {
-                        self.builder_type_doc = Some(*assign.right);
-                        self.doc = true;
-                        Ok(())
-                    }
-                    "build_method_doc" => {
-                        self.build_method_doc = Some(*assign.right);
-                        self.doc = true;
-                        Ok(())
-                    }
+                    "builder_method_doc" => Err(gen_structure_depracation_error("builder_method", "doc")),
+                    "builder_type_doc" => Err(gen_structure_depracation_error("builder_type", "doc")),
+                    "build_method_doc" => Err(gen_structure_depracation_error("build_method", "doc")),
                     _ => Err(Error::new_spanned(&assign, format!("Unknown parameter {:?}", name))),
                 }
             }
@@ -691,6 +743,18 @@ impl TypeBuilderAttr {
                         }
                         Ok(())
                     }
+                    "builder_method" => {
+                        for arg in call.args {
+                            self.builder_method.apply_meta(arg)?;
+                        }
+                        Ok(())
+                    }
+                    "builder_type" => {
+                        for arg in call.args {
+                            self.builder_type.apply_meta(arg)?;
+                        }
+                        Ok(())
+                    }
                     "build_method" => {
                         for arg in call.args {
                             self.build_method.apply_meta(arg)?;
