@@ -240,9 +240,9 @@ fn test_skip() {
 fn test_docs() {
     #[derive(TypedBuilder)]
     #[builder(
-        builder_method_doc = "Point::builder() method docs",
-        builder_type_doc = "PointBuilder type docs",
-        build_method_doc = "PointBuilder.build() method docs"
+        builder_method(doc = "Point::builder() method docs"),
+        builder_type(doc = "PointBuilder type docs"),
+        build_method(doc = "PointBuilder.build() method docs")
     )]
     struct Point {
         #[allow(dead_code)]
@@ -304,8 +304,8 @@ fn test_builder_type_stability_with_other_generics() {
         y: Y,
     }
 
-    impl<X: Default, Y, Y_> FooBuilder<((), Y_), X, Y> {
-        fn x_default(self) -> FooBuilder<((X,), Y_), X, Y> {
+    impl<X: Default, Y, Y_> FooBuilder<X, Y, ((), Y_)> {
+        fn x_default(self) -> FooBuilder<X, Y, ((X,), Y_)> {
             self.x(X::default())
         }
     }
@@ -347,14 +347,14 @@ fn test_builder_type_with_default_on_generic_type() {
         m: M,
     }
 
-    impl<'a, X, Y: Default, M, X_, Y_, M_> FooBuilder<'a, (X_, Y_, (), M_), X, Y, usize, M> {
-        fn z_default(self) -> FooBuilder<'a, (X_, Y_, (usize,), M_), X, Y, usize, M> {
+    impl<'a, X, Y: Default, M, X_, Y_, M_> FooBuilder<'a, X, Y, usize, M, (X_, Y_, (), M_)> {
+        fn z_default(self) -> FooBuilder<'a, X, Y, usize, M, (X_, Y_, (usize,), M_)> {
             self.z(usize::default())
         }
     }
 
-    impl<'a, X, Y: Default, Z: Default, X_, Y_, Z_> FooBuilder<'a, (X_, Y_, Z_, ()), X, Y, Z, ()> {
-        fn m_default(self) -> FooBuilder<'a, (X_, Y_, Z_, ((),)), X, Y, Z, ()> {
+    impl<'a, X, Y: Default, Z: Default, X_, Y_, Z_> FooBuilder<'a, X, Y, Z, (), (X_, Y_, Z_, ())> {
+        fn m_default(self) -> FooBuilder<'a, X, Y, Z, (), (X_, Y_, Z_, ((),))> {
             self.m(())
         }
     }
@@ -574,3 +574,100 @@ fn test_build_method() {
 
     assert!(Foo::builder().x(1).__build() == Foo { x: 1 });
 }
+
+#[test]
+fn test_builder_method() {
+    #[derive(PartialEq, TypedBuilder)]
+    #[builder(builder_method(vis="", name=__builder))]
+    struct Foo {
+        x: i32,
+    }
+
+    assert!(Foo::__builder().x(1).build() == Foo { x: 1 });
+}
+
+#[test]
+fn test_builder_type() {
+    #[derive(PartialEq, TypedBuilder)]
+    #[builder(builder_type(vis="", name=__FooBuilder))]
+    struct Foo {
+        x: i32,
+    }
+
+    let builder: __FooBuilder<_> = Foo::builder();
+    assert!(builder.x(1).build() == Foo { x: 1 });
+}
+
+#[test]
+fn test_default_builder_type() {
+    #[derive(Debug, PartialEq, TypedBuilder)]
+    #[builder(builder_method(vis = ""), builder_type(name = InnerBuilder), build_method(into = Outer))]
+    struct Inner {
+        a: i32,
+        b: i32,
+    }
+
+    #[derive(Debug, PartialEq)]
+    struct Outer(Inner);
+
+    impl Outer {
+        pub fn builder() -> InnerBuilder {
+            Inner::builder()
+        }
+    }
+
+    impl From<Inner> for Outer {
+        fn from(value: Inner) -> Self {
+            Self(value)
+        }
+    }
+
+    let outer = Outer::builder().a(3).b(5).build();
+    assert_eq!(outer, Outer(Inner { a: 3, b: 5 }));
+}
+
+#[test]
+fn test_into_set_generic_impl_from() {
+    #[derive(TypedBuilder)]
+    #[builder(build_method(into))]
+    struct Foo {
+        value: i32,
+    }
+
+    #[derive(Debug, PartialEq)]
+    struct Bar {
+        value: i32,
+    }
+
+    impl From<Foo> for Bar {
+        fn from(value: Foo) -> Self {
+            Self { value: value.value }
+        }
+    }
+
+    let bar: Bar = Foo::builder().value(42).build();
+    assert_eq!(bar, Bar { value: 42 });
+}
+
+#[test]
+fn test_into_set_generic_impl_into() {
+    #[derive(TypedBuilder)]
+    #[builder(build_method(into))]
+    struct Foo {
+        value: i32,
+    }
+
+    #[derive(Debug, PartialEq)]
+    struct Bar {
+        value: i32,
+    }
+
+    impl Into<Bar> for Foo {
+        fn into(self) -> Bar {
+            Bar { value: self.value }
+        }
+    }
+
+    let bar: Bar = Foo::builder().value(42).build();
+    assert_eq!(bar, Bar { value: 42 });
+}
